module Feature.Generation.Links ( genLink ) where

import Core.Types
import Feature.Generation.Shared (rIndex)

import Control.Monad.Reader (ask, lift, liftIO)
import Data.Char (toLower, toUpper)
import System.IO
import System.Random

-- Generates a string containing randomly generated and capitalized
-- characters. The number of characters used is defined in the global config.dhall.
genLink :: PurrAction String
genLink = do 
  linkLength <- confLinkLength
  genLink' linkLength ""
  where
    genLink' 0 cs = return cs
    genLink' d cs = do
      res <- liftIO $ randChar
      fin <- liftIO $ randCapitalization res
      genLink' (d - 1) (cs <> (fin:[]))

-- Defines the valid range of characters to be used when generating links.
-- This consists of all lowercase Latin alphabet characters and the 
-- numbers 1 through 9.
validChars :: [Char]
validChars = ['a'..'z'] <> ['1'..'9']

randChar :: IO Char
randChar = rIndex validChars

randCapitalization :: Char -> IO Char
randCapitalization c = do 
  capRand <- randomRIO (0,1)
  return $ go capRand c
    where
      go :: Int -> Char -> Char
      go r c 
        | r == 0 = toLower c
        | r == 1 = toUpper c
        | otherwise = c  

-- Helper function in our PurrAction monad transformer stack to read
-- the desired length for generated links.
confLinkLength :: PurrAction Int
confLinkLength = do
  conf  <- lift ask
  return $ linkLength conf
